/*
 * Copyright (c) 2002-2003, University of New South Wales
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * Copyright (c) 2007 Open Kernel Labs, Inc. (Copyright Holder).
 * All rights reserved.
 *
 * 1. Redistribution and use of OKL4 (Software) in source and binary
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *     (a) Redistributions of source code must retain this clause 1
 *         (including paragraphs (a), (b) and (c)), clause 2 and clause 3
 *         (Licence Terms) and the above copyright notice.
 *
 *     (b) Redistributions in binary form must reproduce the above
 *         copyright notice and the Licence Terms in the documentation and/or
 *         other materials provided with the distribution.
 *
 *     (c) Redistributions in any form must be accompanied by information on
 *         how to obtain complete source code for:
 *        (i) the Software; and
 *        (ii) all accompanying software that uses (or is intended to
 *        use) the Software whether directly or indirectly.  Such source
 *        code must:
 *        (iii) either be included in the distribution or be available
 *        for no more than the cost of distribution plus a nominal fee;
 *        and
 *        (iv) be licensed by each relevant holder of copyright under
 *        either the Licence Terms (with an appropriate copyright notice)
 *        or the terms of a licence which is approved by the Open Source
 *        Initative.  For an executable file, "complete source code"
 *        means the source code for all modules it contains and includes
 *        associated build and other files reasonably required to produce
 *        the executable.
 *
 * 2. THIS SOFTWARE IS PROVIDED ``AS IS'' AND, TO THE EXTENT PERMITTED BY
 * LAW, ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT, ARE DISCLAIMED.  WHERE ANY WARRANTY IS
 * IMPLIED AND IS PREVENTED BY LAW FROM BEING DISCLAIMED THEN TO THE
 * EXTENT PERMISSIBLE BY LAW: (A) THE WARRANTY IS READ DOWN IN FAVOUR OF
 * THE COPYRIGHT HOLDER (AND, IN THE CASE OF A PARTICIPANT, THAT
 * PARTICIPANT) AND (B) ANY LIMITATIONS PERMITTED BY LAW (INCLUDING AS TO
 * THE EXTENT OF THE WARRANTY AND THE REMEDIES AVAILABLE IN THE EVENT OF
 * BREACH) ARE DEEMED PART OF THIS LICENCE IN A FORM MOST FAVOURABLE TO
 * THE COPYRIGHT HOLDER (AND, IN THE CASE OF A PARTICIPANT, THAT
 * PARTICIPANT). IN THE LICENCE TERMS, "PARTICIPANT" INCLUDES EVERY
 * PERSON WHO HAS CONTRIBUTED TO THE SOFTWARE OR WHO HAS BEEN INVOLVED IN
 * THE DISTRIBUTION OR DISSEMINATION OF THE SOFTWARE.
 *
 * 3. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR ANY OTHER PARTICIPANT BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*                
 * Description:   VAS implementation 
 */                

#include <l4.h>
#include <debug.h>                      /* for UNIMPLEMENTED    */
#include <arch/tlb.h>
#include <cpu/config.h>
#include <arch/mipsregs.h>
#include <arch/context.h>


void SECTION (".init") init_tlb(void)
{
    word_t i;
    TRACEF("Initialize TLB\n");

    __asm__ __volatile__ (
        "mtc0 %0,"STR(CP0_PAGEMASK)"\n\t"
        "mtc0 $0,"STR(CP0_WIRED)"\n\t"
        :
        : "r" (CONFIG_MIPS_PAGEMASK_4K)
    );

    /* Insert invalid entries */
    for (i=0; i<CONFIG_MIPS_TLB_SIZE; i++) {
        __asm__ __volatile__ (
            "mtc0 %0,"STR(CP0_INDEX)"\n\t"
            MTC0" %0,"STR(CP0_ENTRYHI)"\n\t"
            MTC0" $0,"STR(CP0_ENTRYLO0)"\n\t"
            MTC0" $0,"STR(CP0_ENTRYLO1)"\n\t"
            "nop;nop;nop;\n\t"
            "tlbwi\n\t"
            :
            : "r" (i)
        );
    }

    __asm__ __volatile__ (
        "nop;nop;nop;\n\t"
        "mtc0 $0,"STR(CP0_ENTRYHI)"\n\t"
    );
}

void mips64_dump_tlb()
{
/*lint -e529 save is hidden by asm */
    word_t i, save;
    printf("-------------------------TLB DUMP-----------------------------\n");

#if defined(CONFIG_IS_32BIT)
UNIMPLEMENTED();
i = save = 0;
#elif defined(CONFIG_IS_64BIT)
    __asm__ __volatile__ (
        "dmfc0  %0, "STR(CP0_ENTRYHI)"\n\t"
        : "=r" (save)
    );

    for (i=0; i<CONFIG_MIPS_TLB_SIZE; i++)
    {
        word_t hi = 0, lo0 = 0, lo1 = 0;
        __asm__ __volatile__ (
            "mtc0 %3,"STR(CP0_INDEX)"\n\t"
            "nop;nop;nop;tlbr;nop;nop;nop;nop\n\t"
            "dmfc0 %0,"STR(CP0_ENTRYHI)"\n\t"
            "dmfc0 %1,"STR(CP0_ENTRYLO0)"\n\t"
            "dmfc0 %2,"STR(CP0_ENTRYLO1)"\n\t"
            : "=r" (hi), "=r" (lo0), "=r" (lo1)
            : "r" (i)
        );
        printf("%2d: [%p] %08lx / %08lx\n", i, hi, lo0, lo1);
    }

    __asm__ __volatile__ (
        "dmtc0  %0, "STR(CP0_ENTRYHI)"\n\t"
        : : "r" (save)
    );

    printf("-------------------------TLB DUMP-----------------------------\n");
#endif
}

extern "C" void mips64_dump_memory(mips_irq_context_t *context)
{
    unsigned int i;

    mips64_dump_tlb();

    printf("-------------------------EPC DUMP-----------------------------\n");
    printf("EPC = %16lx\n", context->epc);
    for (i=0; i<32; i+= 4)
    {
        printf("%c [%16lx] = 0x%08lx\n",i==24? '*' : ' ', context->epc-24+i, *(unsigned int*)(context->epc-24+i));
    }
}
